local L = AceLibrary("AceLocale-2.2"):new("MobMitigation")
MobMitigation = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0")
MobMitigation:RegisterDB("MobMitigationDB")
local tablet = AceLibrary("Tablet-2.0")
local parser = ParserLib:GetInstance("1.1")
local BC = AceLibrary("Babble-Class-2.2")
local BS = AceLibrary("Babble-Spell-2.2")
local deformat = AceLibrary("Deformat-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local skills = {}
local data = {}
local showboxes = {}
local options = {
	type = 'group',
	args = {
		savedata = {
			type = 'toggle',
			name = L["Toggle Data Saving"],
			desc = L["Toggle Data Saving"],
			get = function()
				return MobMitigation.db.profile.savedata
			end,
			set = function(v)
				MobMitigation.db.profile.savedata = v
				if v==true then
					data = MobMitigation.db.account
				else
					data = {}
					MobMitigation.db.account = {}
				end
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
		},
		layout = {
			type = 'toggle',
			name = L["Toggle Layout"],
			desc = L["Toggle between Vertical and Horizontal layout"],
			get = function()
				return MobMitigation.db.profile.layout
			end,
			set = function(v)
				MobMitigation.db.profile.layout = v
				MobMitigation:UpdateLayout()
			end,
			map = { [false] = "Vertical", [true] = "Horizontal" },
		},
		lock = {
			type = 'toggle',
			name = L["Toggle Lock"],
			desc = L["Toggle Lock"],
			get = function()
				return MobMitigationDragFrame:IsShown()
			end,
			set = function(v)
				if v==true then
					MobMitigationDragFrame:Show()
				else
					MobMitigationDragFrame:Hide()
				end
			end,
			map = { [false] = "Locked", [true] = "Unlocked" },
		},
		percentage = {
			type = 'toggle',
			name = L["Toggle Percentage Display"],
			desc = L["Toggle Percentage Display"],
			get = function()
				return MobMitigation.db.profile.percentage
			end,
			set = function(v)
				MobMitigation.db.profile.percentage = v
				MobMitigation:Update()
			end,
			map = { [false] = "Actual", [true] = "Percentage" },
		},
		tooltip = {
			type = 'toggle',
			name = L["Toggle Tooltip Display"],
			desc = L["Toggle Tooltip Display"],
			get = function()
				return MobMitigation.db.profile.tooltip
			end,
			set = function(v)
				MobMitigation.db.profile.tooltip = v
				MobMitigation:UpdateLayout()
			end,
			map = { [false] = "Disabled", [true] = "Enabled" },
		},
	},
}
function MobMitigation:OnInitialize()
	MobMitigation:RegisterChatCommand({"/mobmitigation", "/mobmit"}, options)
	if MobMitigation.db.profile.savedata == nil then
		MobMitigation.db.profile.savedata = true
	end
	if MobMitigation.db.profile.layout == nil then
		MobMitigation.db.profile.layout = false
	end
	if MobMitigation.db.profile.percentage == nil then
		MobMitigation.db.profile.percentage = true
	end
	if MobMitigation.db.profile.tooltip == nil then
		MobMitigation.db.profile.tooltip = true
	end
	
	if MobMitigation.db.profile.savedata then
		data = MobMitigation.db.account
	end
	
	local class = UnitClass("player")
	if class == BC["Mage"] then
		skills = {
			[BS["Arcane Explosion"]] = L["Arcane"],
			[BS["Arcane Missiles"]] = L["Arcane"],
			[BS["Arcane Blast"]] = L["Arcane"],
			[BS["Fire Blast"]] = L["Fire"],
			[BS["Fireball"]] = L["Fire"],
			[BS["Pyroblast"]] = L["Fire"],
			[BS["Scorch"]] = L["Fire"],
			[BS["Dragon's Breath"]] = L["Fire"],
			[BS["Frostbolt"]] = L["Frost"],
			[BS["Cone of Cold"]] = L["Frost"],
			[BS["Ice Lance"]] = L["Frost"],
			[BS["Frost Nova"]] = L["Frost"],
			[BS["Shoot"]] = true,
		}
		showboxes = {"Fire","Frost","Arcane"}
	elseif class == BC["Warlock"] then
		skills = {
			[BS["Immolate"]] = L["Fire"],
			[BS["Conflagrate"]] = L["Fire"],
			[BS["Searing Pain"]] = L["Fire"],
			[BS["Soul Fire"]] = L["Fire"],
			[BS["Incinerate"]] = L["Fire"],
			[BS["Shadow Bolt"]] = L["Shadow"],
			[BS["Shadowburn"]] = L["Shadow"],
			[BS["Shoot"]] = true,
		}
		showboxes = {"Shadow","Fire"}
	elseif class == BC["Priest"] then
		skills = {
			[BS["Mind Blast"]] = L["Shadow"],
			[BS["Mind Flay"]] = L["Shadow"],
			[BS["Mana Burn"]] = L["Shadow"],
			[BS["Shoot"]] = true,
		}
		showboxes = {"Shadow"}
	elseif class == BC["Druid"] then
		skills = {
			[BS["Wrath"]] = L["Nature"],
			[BS["Moonfire"]] = L["Arcane"],
			[BS["Starfire"]] = L["Arcane"],
		}
		showboxes = {"Nature","Arcane","Armor"}
	elseif class == BC["Shaman"] then
		skills = {
			[BS["Frost Shock"]] = L["Frost"],
			[BS["Chain Lightning"]] = L["Nature"],
			[BS["Lightning Bolt"]] = L["Nature"],
		}
		showboxes = {"Nature","Frost","Armor"}
	elseif class == BC["Hunter"] then
		skills = {
			[BS["Auto Shot"]] = true,
		}
		showboxes = {"Armor"}
	else
		skills = {} --everyone else usin' autoattack!
		showboxes = {"Armor"}
	end
	
	CreateFrame("Frame", "MobMitigationFrame",UIParent)
	MobMitigationFrame:SetFrameStrata("HIGH")
	MobMitigationFrame:SetWidth(20)
	MobMitigationFrame:SetHeight(120)
	MobMitigationFrame:SetPoint("CENTER",0,0)
	MobMitigationFrame:SetMovable(true)
	
	CreateFrame("Frame", "MobMitigationDragFrame",MobMitigationFrame)
	MobMitigationDragFrame:SetFrameStrata("DIALOG")
	MobMitigationDragFrame:SetWidth(20)
	MobMitigationDragFrame:SetHeight(120)
	MobMitigationDragFrame:SetPoint("TOPLEFT",MobMitigationFrame)
	MobMitigationDragFrame:EnableMouse(true)
	MobMitigationDragFrame:SetScript("OnMouseDown", function() MobMitigationFrame:StartMoving() end)
	MobMitigationDragFrame:SetScript("OnMouseUp", function() MobMitigationFrame:StopMovingOrSizing() end)
	MobMitigationDragFrame:Hide()
	
	
	CreateFrame("Frame", "MobMitigationFrameArcane",MobMitigationFrame)
	MobMitigationFrameArcane:SetFrameStrata("MEDIUM")
	MobMitigationFrameArcane:SetWidth(20)
	MobMitigationFrameArcane:SetHeight(20)
	MobMitigationFrameArcane:CreateTexture("MobMitigationFrameArcaneTexture","ARTWORK")
	MobMitigationFrameArcaneTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Arcane.tga")
	MobMitigationFrameArcaneTexture:SetAllPoints(MobMitigationFrameArcane)
	MobMitigationFrameArcane:Hide()
	MobMitigationFrameArcane.texture = MobMitigationFrameArcaneTexture
	MobMitigationFrameArcane:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,0)
	MobMitigationFrameArcane:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameArcane,'children',function() MobMitigation:Tooltip(L["Arcane"]) end)
	tablet:SetColor(MobMitigationFrameArcane, 0.3, 0.25, 0.3)
	dewdrop:Register(MobMitigationFrameArcane,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameArcaneText","HIGH")
	MobMitigationFrameArcaneText:SetPoint("CENTER",MobMitigationFrameArcane,0,0)
	MobMitigationFrameArcaneText:SetFontObject("GameFontHighlight")
	MobMitigationFrameArcaneText:SetTextColor(1, 1, 1)
	MobMitigationFrameArcaneText:Hide()
		
	CreateFrame("Frame", "MobMitigationFrameFire",MobMitigationFrame)
	MobMitigationFrameFire:SetFrameStrata("MEDIUM")
	MobMitigationFrameFire:SetWidth(20)
	MobMitigationFrameFire:SetHeight(20)
	MobMitigationFrameFire:CreateTexture("MobMitigationFrameFireTexture","ARTWORK")
	MobMitigationFrameFireTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Fire.tga")
	MobMitigationFrameFireTexture:SetAllPoints(MobMitigationFrameFire)
	MobMitigationFrameFire:Hide()
	MobMitigationFrameFire.texture = MobMitigationFrameFireTexture
	MobMitigationFrameFire:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,-20)
	MobMitigationFrameFire:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameFire,'children',function() MobMitigation:Tooltip(L["Fire"]) end)
	tablet:SetColor(MobMitigationFrameFire, 0.4, 0, 0)
	dewdrop:Register(MobMitigationFrameFire,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameFireText","HIGH")
	MobMitigationFrameFireText:SetPoint("CENTER",MobMitigationFrameFire,0,0)
	MobMitigationFrameFireText:SetFontObject("GameFontHighlight")
	MobMitigationFrameFireText:SetTextColor(1, 1, 1)
	MobMitigationFrameFireText:Hide()
	
	CreateFrame("Frame", "MobMitigationFrameFrost",MobMitigationFrame)
	MobMitigationFrameFrost:SetFrameStrata("MEDIUM")
	MobMitigationFrameFrost:SetWidth(20)
	MobMitigationFrameFrost:SetHeight(20)
	MobMitigationFrameFrost:CreateTexture("MobMitigationFrameFrostTexture","ARTWORK")
	MobMitigationFrameFrostTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Frost.tga")
	MobMitigationFrameFrostTexture:SetAllPoints(MobMitigationFrameFrost)
	MobMitigationFrameFrost:Hide()
	MobMitigationFrameFrost.texture = MobMitigationFrameFrostTexture
	MobMitigationFrameFrost:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,-40)
	MobMitigationFrameFrost:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameFrost,'children',function() MobMitigation:Tooltip(L["Frost"]) end)
	tablet:SetColor(MobMitigationFrameFrost, 0, 0, 0.4)
	dewdrop:Register(MobMitigationFrameFrost,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameFrostText","HIGH")
	MobMitigationFrameFrostText:SetPoint("CENTER",MobMitigationFrameFrost,0,0)
	MobMitigationFrameFrostText:SetFontObject("GameFontHighlight")
	MobMitigationFrameFrostText:SetTextColor(1, 1, 1)
	MobMitigationFrameFrostText:Hide()
	
	CreateFrame("Frame", "MobMitigationFrameShadow",MobMitigationFrame)
	MobMitigationFrameShadow:SetFrameStrata("MEDIUM")
	MobMitigationFrameShadow:SetWidth(20)
	MobMitigationFrameShadow:SetHeight(20)
	MobMitigationFrameShadow:CreateTexture("MobMitigationFrameShadowTexture","ARTWORK")
	MobMitigationFrameShadowTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Shadow.tga")
	MobMitigationFrameShadowTexture:SetAllPoints(MobMitigationFrameShadow)
	MobMitigationFrameShadow:Hide()
	MobMitigationFrameShadow.texture = MobMitigationFrameShadowTexture
	MobMitigationFrameShadow:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,-60)
	MobMitigationFrameShadow:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameShadow,'children',function() MobMitigation:Tooltip(L["Shadow"]) end)
	tablet:SetColor(MobMitigationFrameShadow, 0.3, 0.25, 0.3)
	dewdrop:Register(MobMitigationFrameShadow,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameShadowText","HIGH")
	MobMitigationFrameShadowText:SetPoint("CENTER",MobMitigationFrameShadow,0,0)
	MobMitigationFrameShadowText:SetFontObject("GameFontHighlight")
	MobMitigationFrameShadowText:SetTextColor(1, 1, 1)
	MobMitigationFrameShadowText:Hide()
	
	CreateFrame("Frame", "MobMitigationFrameNature",MobMitigationFrame)
	MobMitigationFrameNature:SetFrameStrata("MEDIUM")
	MobMitigationFrameNature:SetWidth(20)
	MobMitigationFrameNature:SetHeight(20)
	MobMitigationFrameNature:CreateTexture("MobMitigationFrameNatureTexture","ARTWORK")
	MobMitigationFrameNatureTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Nature.tga")
	MobMitigationFrameNatureTexture:SetAllPoints(MobMitigationFrameNature)
	MobMitigationFrameNature:Hide()
	MobMitigationFrameNature.texture = MobMitigationFrameNatureTexture
	MobMitigationFrameNature:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,-80)
	MobMitigationFrameNature:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameNature,'children',function() MobMitigation:Tooltip(L["Nature"]) end)
	tablet:SetColor(MobMitigationFrameNature, 0, 0.4, 0)
	dewdrop:Register(MobMitigationFrameNature,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameNatureText","HIGH")
	MobMitigationFrameNatureText:SetPoint("CENTER",MobMitigationFrameNature,0,0)
	MobMitigationFrameNatureText:SetFontObject("GameFontHighlight")
	MobMitigationFrameNatureText:SetTextColor(1, 1, 1)
	MobMitigationFrameNatureText:Hide()
	
	CreateFrame("Frame", "MobMitigationFrameArmor",MobMitigationFrame)
	MobMitigationFrameArmor:SetFrameStrata("MEDIUM")
	MobMitigationFrameArmor:SetWidth(20)
	MobMitigationFrameArmor:SetHeight(20)
	MobMitigationFrameArmor:CreateTexture("MobMitigationFrameArmorTexture","ARTWORK")
	MobMitigationFrameArmorTexture:SetTexture("Interface\\Addons\\MobMitigation\\images\\Armor.tga")
	MobMitigationFrameArmorTexture:SetAllPoints(MobMitigationFrameArmor)
	MobMitigationFrameArmor:Hide()
	MobMitigationFrameArmor.texture = MobMitigationFrameArmorTexture
	MobMitigationFrameArmor:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,-100)
	MobMitigationFrameArmor:SetAlpha(0.75)
	tablet:Register(MobMitigationFrameArmor,'children',function() MobMitigation:Tooltip(L["Armor"]) end)
	dewdrop:Register(MobMitigationFrameArmor,'children',function() dewdrop:FeedAceOptionsTable(options) end)
	MobMitigationFrame:CreateFontString("MobMitigationFrameArmorText","HIGH")
	MobMitigationFrameArmorText:SetPoint("CENTER",MobMitigationFrameArmor,0,0)
	MobMitigationFrameArmorText:SetFontObject("GameFontHighlight")
	MobMitigationFrameArmorText:SetTextColor(1, 1, 1)
	MobMitigationFrameArmorText:Hide()
	
end
function MobMitigation:OnEnable()
	parser:RegisterEvent("MobMitigation", "CHAT_MSG_COMBAT_SELF_HITS", MobMitigation.Hit)
	parser:RegisterEvent("MobMitigation", "CHAT_MSG_SPELL_SELF_DAMAGE", MobMitigation.Hit)
	MobMitigation:RegisterEvent("PLAYER_TARGET_CHANGED")
	MobMitigationFrame:Show()
	MobMitigation:UpdateLayout()
end
function MobMitigation:OnDisable()
	parser:UnregisterAllEvents("MobMitigation")
	MobMitigation:UnregisterAllEvents()
	MobMitigationFrame:Hide()
end
function MobMitigation:Hit(info)
	--cheats for wanding and autoshot.
	--note: wanding is scratched for now, partial resist events arent making their way through
	if info.type == "unknown" then
		info.skill, info.victim, info.amount = deformat(info.message, SPELLLOGSELFOTHER)
		info.type = "hit"
		--wand:info.skill, info.victim, info.amount, info.element = deformat(info.message, SPELLLOGSCHOOLSELFOTHER)
		--[[ wand stuff.
		if info.skill and string.find(info.message,L["resisted"]) then
			--partial resist.  track it down!
			local i=string.find(info.message,"resisted")
			local j=string.find(info.message," ",i-9)
			info.amountResist=tonumber(string.sub(info.message1,j+2,i-2))
		end
		if (not info.skill) then
			--wand shot full resisted?
			info.skill, info.victim = deformat(info.message, SPELLRESISTSELFOTHER)
			info.school = "?"--what element is this? find it
			info.type = "miss"
		end]]
		if (not info.skill) or string.find(info.message,L["absorbed"]) or string.find(info.message,L["vulnerability"]) or string.find(info.message,L["glancing"]) then
			return
		end
	end
	if MobMitigation:IsDebugging() then 
		for k,v in pairs(info) do
			MobMitigation:Debug(k,":",v)
		end
	end
	if info.type == "unknown" then return end
	if info.isDOT or info.isCrushing or info.isGlancing or info.amountVulnerable or info.amountBlock then
		return
	end
	if info.victim == UnitName("target") then
		info.victimLevel = UnitLevel("target")
	else
		return
	end
	if info.element or (info.type=="miss" and info.missType == "resist") then
		if (not info.element) then
			if info.type == "miss" then
				info.element = skills[info.skill]
			end
			if (not info.element) then
				return
			end
		end
		if type(info.element)~="string" then return end
		if data[info.victim..";"..info.victimLevel..";"..info.element] == nil then
			data[info.victim..";"..info.victimLevel..";"..info.element] = "0;0;0;0;0"
		end
		if info.type=="miss" then
			local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[info.victim..";"..info.victimLevel..";"..info.element], "%d;%d;%d;%d;%d")
			fullresist = fullresist+1
			data[info.victim..";"..info.victimLevel..";"..info.element] = nonresist..";"..quarterresist..";"..halfresist..";"..threequarterresist..";"..fullresist
			MobMitigation:Debug("incremented full resists")
		elseif skills[info.skill] then
			if info.amountAbsorb then
				info.amount = info.amount + info.amountAbsorb
			end
			if (not info.amountResist) then
				local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[info.victim..";"..info.victimLevel..";"..info.element], "%d;%d;%d;%d;%d")
				nonresist = nonresist+1
				data[info.victim..";"..info.victimLevel..";"..info.element] = nonresist..";"..quarterresist..";"..halfresist..";"..threequarterresist..";"..fullresist
				MobMitigation:Debug("incremented nonresists")
			else
				if info.amountResist > (info.amount*1.25) then
					local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[info.victim..";"..info.victimLevel..";"..info.element], "%d;%d;%d;%d;%d")
					threequarterresist = threequarterresist+1
					data[info.victim..";"..info.victimLevel..";"..info.element] = nonresist..";"..quarterresist..";"..halfresist..";"..threequarterresist..";"..fullresist
					MobMitigation:Debug("incremented 3/4th")
				elseif (info.amountResist >= (info.amount*0.75)) and (info.amountResist <= (info.amount*1.25)) then
					local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[info.victim..";"..info.victimLevel..";"..info.element], "%d;%d;%d;%d;%d")
					halfresist = halfresist+1
					data[info.victim..";"..info.victimLevel..";"..info.element] = nonresist..";"..quarterresist..";"..halfresist..";"..threequarterresist..";"..fullresist
					MobMitigation:Debug("incremented half")
				else
					local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[info.victim..";"..info.victimLevel..";"..info.element], "%d;%d;%d;%d;%d")
					quarterresist = quarterresist+1
					data[info.victim..";"..info.victimLevel..";"..info.element] = nonresist..";"..quarterresist..";"..halfresist..";"..threequarterresist..";"..fullresist
					MobMitigation:Debug("incremented 1/4")
				end
			end
		end
	elseif info.type == "hit" and (not info.isCrit) and MobMitigationFrameArmor:IsShown() and info.amount then
		if skills[info.skill] then
			MobMitigation:Debug("ranged")
			local speed, lowDmg, hiDmg = UnitRangedDamage("player")
			local averagerangedhit = (lowDmg + hiDmg) / 2;
			if data[info.victim..";"..info.victimLevel..";".."Armor"] == nil then
				data[info.victim..";"..info.victimLevel..";".."Armor"] = "0;0"
			end
			if info.amountAbsorb then
				info.amount = info.amount + info.amountAbsorb
			end
			local percentage, numhits = deformat(data[info.victim..";"..info.victimLevel..";".."Armor"], "%d;%d")
			percentage = percentage/100000
			percentage = math.floor((( (1-(info.amount/averagerangedhit)) + (percentage * numhits) )/(numhits+1))*100000)
			numhits = numhits+1
			data[info.victim..";"..info.victimLevel..";".."Armor"] = percentage..";"..numhits
		elseif tonumber(info.skill) then
			MobMitigation:Debug("melee")
			local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player")
			if ( ( offlowDmg ~= offhiDmg ) and ( offlowDmg ~= 0 or offhiDmg ~= 0 ) ) then
				if ( tonumber(info.amount) < tonumber(offhiDmg) ) then return; end
			end
			local averagemeleehit = (lowDmg + hiDmg) * percentmod / 2;
			if data[info.victim..";"..info.victimLevel..";".."Armor"] == nil then
				data[info.victim..";"..info.victimLevel..";".."Armor"] = "0;0"
			end
			if info.amountAbsorb then
				info.amount = info.amount + info.amountAbsorb
			end
			local percentage, numhits = deformat(data[info.victim..";"..info.victimLevel..";".."Armor"], "%d;%d")
			percentage = percentage/100000
			percentage = math.floor((( (1-(info.amount/averagemeleehit)) + (percentage * numhits) )/(numhits+1))*100000)
			numhits = numhits+1
			data[info.victim..";"..info.victimLevel..";".."Armor"] = percentage..";"..numhits
		end
	end
	MobMitigation:Update()
end
function MobMitigation:PLAYER_TARGET_CHANGED()
	if UnitName("target") then
		MobMitigation:Update()
	else
		MobMitigationFrame:Hide()
	end
end
function MobMitigation:Update()
	local targetname = UnitName("target")
	local targetlevel = UnitLevel("target")
	if (not targetname) then return end
	for k,v in pairs (showboxes) do
		if data[targetname..";"..targetlevel..";"..L[v]] then
			if v=="Armor" then
				local percentage, numhits = deformat(data[targetname..";"..targetlevel..";"..v], "%d;%d")
				percentage = percentage/100000
				if percentage < 0 then
					percentage=0
				end
				if MobMitigation.db.profile.percentage then
					getglobal("MobMitigationFrame"..v.."Text"):SetText((math.floor(percentage*10000)/100).."%")
				else
					--this formula taken from TargetArmor by Blackmail
					getglobal("MobMitigationFrame"..v.."Text"):SetText(math.floor(( percentage * ( 85 * targetlevel + 400 ) ) / ( 1 - percentage )))
				end
			else
				local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[targetname..";"..targetlevel..";"..v], "%d;%d;%d;%d;%d")
				local totalcasts = nonresist+quarterresist+halfresist+threequarterresist+fullresist
				if totalcasts == 0 then return end
				local resistancepercent = (0.25*quarterresist)/totalcasts +(0.5*halfresist)/totalcasts +(0.75*threequarterresist)/totalcasts +fullresist/totalcasts
				if MobMitigation.db.profile.percentage then
					getglobal("MobMitigationFrame"..v.."Text"):SetText(math.floor(resistancepercent*100).."%")
				else
					local effectivetargetlevel
					if targetlevel == -1 then
						effectivetargetlevel=UnitLevel("player")+3--good assumption, eh? lets hope it's not 10 levels above the player.
					else
						effectivetargetlevel=targetlevel
					end
					getglobal("MobMitigationFrame"..v.."Text"):SetText(math.floor((5*effectivetargetlevel)*(resistancepercent*(4/3))))
				end
			end
		else
			getglobal("MobMitigationFrame"..v.."Text"):SetText("")
		end
	end
	MobMitigationFrame:Show()
end
function MobMitigation:UpdateLayout()
	local numframes=0
	if (not MobMitigation.db.profile.layout) then--vertical
		for k,v in pairs(showboxes) do
			local frame = getglobal("MobMitigationFrame"..showboxes[k])
			if MobMitigation.db.profile.tooltip then
				frame:EnableMouse(true)
			else
				frame:EnableMouse(false)
			end
			frame:Show()
			frame:ClearAllPoints()
			frame:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",0,(-20*numframes))
			getglobal("MobMitigationFrame"..showboxes[k].."Text"):Show()
			numframes=numframes+1
		end
		MobMitigationFrame:SetWidth(20)
		MobMitigationFrame:SetHeight(20*numframes)
		MobMitigationDragFrame:SetWidth(20)
		MobMitigationDragFrame:SetHeight(20*numframes)
	else--horizontal
		for k,v in pairs(showboxes) do
			local frame = getglobal("MobMitigationFrame"..showboxes[k])
			if MobMitigation.db.profile.tooltip then
				frame:EnableMouse(true)
			else
				frame:EnableMouse(false)
			end
			frame:Show()
			frame:ClearAllPoints()
			frame:SetPoint("TOPLEFT",MobMitigationFrame,"TOPLEFT",(20*numframes),0)
			getglobal("MobMitigationFrame"..showboxes[k].."Text"):Show()
			numframes=numframes+1
		end
		MobMitigationFrame:SetWidth(20*numframes)
		MobMitigationFrame:SetHeight(20)
		MobMitigationDragFrame:SetWidth(20*numframes)
		MobMitigationDragFrame:SetHeight(20)
	end
	MobMitigationFrame:Hide()
	MobMitigation:Update()
end

function MobMitigation:PrintData()
	for k,v in pairs(data) do
		MobMitigation:Debug(k,":",v)
	end
end

function MobMitigation:Tooltip(element)
	local targetname = UnitName("target")
	local targetlevel = UnitLevel("target")
	if (not targetname) then return end
	if (not data[targetname..";"..targetlevel..";"..element]) then
		tablet:SetTitle(L["No Data"])
		return
	end
	tablet:SetTitle(element)
	local cat = tablet:AddCategory('columns', 2)
	if element == L["Armor"] then
		local percentage, numhits = deformat(data[targetname..";"..targetlevel..";"..element], "%d;%d")
		percentage = percentage/100000
		if percentage < 0 then
			percentage=0
		end
		local estimate = math.floor(( percentage * ( 85 * targetlevel + 400 ) ) / ( 1 - percentage ))
		cat:AddLine(
			'text', L["Total Hits"],
			'text2', numhits
		)
		cat:AddLine(
			'text', L["Percentage Mitigated"],
			'text2',((math.floor(percentage*10000)/100).."%")
		)
		cat:AddLine()
		cat:AddLine(
			'text', (L["Estimated"].." "..L["Armor"]),
			'text2', estimate
		)
	else
		local nonresist, quarterresist, halfresist, threequarterresist, fullresist = deformat(data[targetname..";"..targetlevel..";"..element], "%d;%d;%d;%d;%d")
		local totalcasts = nonresist+quarterresist+halfresist+threequarterresist+fullresist
		if totalcasts == 0 then return end
		local resistancepercent = (0.25*quarterresist)/totalcasts +(0.5*halfresist)/totalcasts +(0.75*threequarterresist)/totalcasts +fullresist/totalcasts
		local effectivetargetlevel
		if targetlevel == -1 then
			effectivetargetlevel=UnitLevel("player")+3--good assumption, eh? lets hope it's not 10 levels above the player.
		else
			effectivetargetlevel=targetlevel
		end
		local estimate = math.floor((5*effectivetargetlevel)*(resistancepercent*(4/3)))
		cat:AddLine(
			'text', L["Total Casts"],
			'text2', totalcasts
		)
		cat:AddLine(
			'text', L["Fully Resisted Casts"],
			'text2', fullresist
		)
		cat:AddLine(
			'text', L["Three-Quarter Resisted Casts"],
			'text2', threequarterresist
		)
		cat:AddLine(
			'text', L["Half Resisted Casts"],
			'text2', halfresist
		)
		cat:AddLine(
			'text', L["Quarter Resisted Casts"],
			'text2', quarterresist
		)
		cat:AddLine(
			'text', L["Non Resisted Casts"],
			'text2', nonresist
		)
		cat:AddLine()
		cat:AddLine(
			'text', L["Percentage Mitigated"],
			'text2', ((math.floor(resistancepercent*10000)/100).."%")
		)
		cat:AddLine(
			'text', (L["Estimated"].." "..element.." "..L["Resistance"]),
			'text2', estimate
		)
	end
end
--for armor, store as percentage and number of hits.  "15675;3" (15.675%, 3 hits)
--data["mobname;moblevel;school"] = "0%resists;25%;50%;75%;100%"

